from streamlit_chitchat.streamlit_chitchat.chitchat import message
import streamlit as st


# example test code
def test1():
    import time
    message('hello', is_user=True, avatar='fun-emoji', seed=100)
    bot = message('hello as well', is_user=False, avatar='bottts', seed=100)
    message('what do you mean?', is_user=True, avatar='images/user.png', seed=100)
    time.sleep(0.3)
    bot.write('actually, i wanted to say sorry')
    message('ok, i understand', is_user=True,
            avatar='https://images.unsplash.com/photo-1539571696357-5a69c17a67c6?ixlib=rb-4.0.3&ixid=M3wxMjA3fDB8MHxwaG90by1wYWdlfHx8fGVufDB8fHx8fA%3D%3D&auto=format&fit=crop&w=1974&q=80')
    message()
    message('hello')
    message('hello as well', is_user=True)
    text = '''Wikipedia is an online encyclopedia written and maintained by a community of volunteers, known as Wikipedians,
             through open collaboration and using a wiki-based editing system called MediaWiki. 
             Wikipedia is the largest and most-read reference work in history, and has consistently been 
             one of the 10 most popular websites.'''
    m1 = message()
    t=''
    for word in text.split(' '):
        t+=word+' '
        m1.write(t)
        time.sleep(0.05)
def test2():
    with st.container():
        # image file
        st.write('\n')
        st.write('\nimage file')
        message('hello ', is_user=True, avatar='images/man.png', background='lightgreen')
        # default avatar (different for bot and user)
        st.write('\n')
        st.write('\ndefault')
        message('hello', is_user=False, avatar='', seed=142, background='lightblue')
        # inline svg
        st.write('\n')
        st.write('\ninline svg')
        svg = '''<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="60px" height="40px" viewBox="0 0 122.88 80.593" enable-background="new 0 0 122.88 80.593" xml:space="preserve"><g><polygon points="122.88,0 122.88,30.82 61.44,80.593 0,30.82 0,0 61.44,49.772 122.88,0"/></g></svg>'''
        message('hello', is_user=False, avatar=svg, seed=142, background='lightblue')
        st.write('\n')
        st.write('\nsvg file')
        message('hello', is_user=False, avatar='images/angle-bottom-icon.svg', seed=142, background='lightblue')

        # no avatar
        st.write('\n')
        st.write('\nno avatar')
        message('hello', is_user=True, background='lightgreen')
        message('hello', is_user=False, background='lightblue')
        # background as rgb color
        st.write('\n')
        st.write('\nrgb background')
        message('hello', is_user=True, background='rgb(50,150,150)')
        message('hello', is_user=False, background='rgb(160,150,141)')
        # background as rgb color with alpha channel
        st.write('\n')
        st.write('\nrgb with alpha channel')
        message('hello', is_user=True, background='rgb(50,150,150,0.5)')
        message('hello', is_user=False, background='rgb(160,150,141,0.5)')
        st.write('\n')
        st.write('\n other styles')
        st.write('\n color, width, margin')
        message('hello', color='rgb(220,20,30)', width='60%', margin='20px')
        st.write('\n other styles')
        st.write('\n font size')
        message('hello', font_size='3rem')
        st.write('\n other styles')
        st.write('\n padding')
        message('hello', padding='30px')


if __name__ == '__main__':
    test1()
    test2()




bot_avatars = ['https://img.uxwing.com/wp-content/themes/uxwing/download/internet-network-technology/robot-icon.svg',
               'https://img.uxwing.com/wp-content/themes/uxwing/download/internet-network-technology/robot-line-icon.svg']
user_avatars = [
    'https://img.uxwing.com/wp-content/themes/uxwing/download/peoples-avatars-thoughts/boy-glasses-icon.svg',
    'https://img.uxwing.com/wp-content/themes/uxwing/download/peoples-avatars-thoughts/man-face-with-glasses-icon.svg',
    'https://img.uxwing.com/wp-content/themes/uxwing/download/peoples-avatars-thoughts/girl-icon.svg',
    'https://img.uxwing.com/wp-content/themes/uxwing/download/peoples-avatars-thoughts/girl-eyeglasses-icon.svg']
dicebear_avatars = ['adventurer', 'adventurer-neutral', 'avataaars', 'avataaars-neutral', 'big-ears',
                    'big-ears-neutral', 'big-smile', 'bottts', 'bottts-neutral', 'croodles', 'croodles-neutral',
                    'fun-emoji', 'icons', 'identicon', 'initials', 'lorelei', 'lorelei-neutral', 'micah', 'miniavs',
                    'open-peeps', 'personas', 'pixel-art', 'pixel-art-neutral', 'shapes', 'thumbs']

i0 = '''<div></div>'''
i1 = '<img src="{image}" alt="" style="border:1px solid transparent;{border_radius};{width};{height};">'
t1 = '''
    <div style="display:flex;{font_size}{font}{color}flex-direction:row-reverse;{margin}">
        {img}
        <div
            style="display:flex;text-align:left;align-items:center;border:1px solid transparent;border-radius:5px;{background}{width}{height}{padding}">
            {message}
        </div>
    </div>
    '''
t2 = '''
    <div style="display:flex;{font_size}{font}{color}flex-direction:row;{margin}">
        {img}
        <div
            style="display:flex;text-align:left;align-items:center;border:1px solid transparent;border-radius:5px;{background}{width}{height}{padding}">
            {message}
        </div>

    </div>
    '''

t3 = '''
     <img src="{image}" alt="blue dot" />
    '''



import os
import streamlit as st
from streamlit_chitchat.streamlit_chitchat.utils import _is_svg, _is_url, encode_image
from streamlit_chitchat.streamlit_chitchat.data import i0, i1, t1, t2, bot_avatars, user_avatars, dicebear_avatars


class message:
    def __init__(self, text=None, element=None, is_user=None, font=None, font_size=None, color=None, background=None,
                 width=None, height=None, avatar=None, avatar_size=None, avatar_mode=None, seed=None, margin=None,
                 padding=None):
        self.element = element if element is not None else st.empty()
        self.is_user = is_user
        self.font = 'Geneva, Verdana, sans-serif' if font is None else font
        self.font_size = '1rem;' if font_size is None else font_size
        self.padding = '4px 10px 4px 10px;' if padding is None else padding
        self.margin = '2px' if margin is None else margin
        self.background = 'rgb(240, 242, 246)' if background is None else background
        self.color = 'rgb(49, 51, 63)' if color is None else color
        self.width = '70%;' if width is None else width
        self.height = '3rem' if height is None else height
        self.ext = 'jpg'
        self.svg = False
        self.avatar_size = avatar_size if avatar_size else '3rem'
        self.avatar_mode = '15%' if avatar_mode == 'square' else '50%'
        if avatar in dicebear_avatars:
            seed = 10 if seed is None else seed
            self.avatar = f'https://api.dicebear.com/5.x/{avatar}/svg?seed={seed}'
        elif _is_url(avatar):
            self.avatar = f'{avatar}'
        elif avatar and os.path.isfile(avatar):
            # svg file
            if avatar.endswith('.svg'):
                avtr = open(avatar, 'r').read()
                if _is_svg(avtr):
                    self.avatar = f'{avtr}'
                    self.svg = True
                else:
                    print(f'{avatar}: does not appear to be an svg file despite of its extension')
                    self.avatar = ''
            # image file
            else:
                ex = os.path.splitext(avatar)[1]
                ex = ex[1:] if ex else ex
                self.ext = 'jpg' if not ex else ex
                fs = f'data:image/{self.ext};base64,'
                self.avatar = f'{fs}{encode_image(avatar)}'
        elif _is_svg(avatar):
            self.avatar = f'{avatar}'
            self.svg = True
        # use default bot/user avatar
        elif avatar == '':
            self.avatar = user_avatars[0] if self.is_user else bot_avatars[0]
        # default case: no avatar
        else:
            self.avatar = None

        if text:
            self.write(text)

    def _message(self, text):
        h1 = t1 if self.is_user else t2
        width = f'width:{self.width};'  # max-width
        height = f'min-height:{self.height};'
        font = f'font-family: {self.font};'  # 'font-family: Geneva, Verdana, sans-serif;'
        font_size = f'font-size: {self.font_size};'
        color = f'color:{self.color};'
        background = f'background-color:{self.background};'
        margin = f'margin:{self.margin};'
        padding = f'padding:{self.padding};'
        border_radius = f'border-radius:{self.avatar_mode};'
        avatar_width = f'width:{self.avatar_size}'
        avatar_height = f'height:{self.avatar_size}'

        img = self.avatar if self.svg else i0 if self.avatar is None else i1.format(image=self.avatar,
                                                                                    border_radius=border_radius,
                                                                                    width=avatar_width,
                                                                                    height=avatar_height)

        ret = h1.format(img=img, message=text, width=width, height=height, font=font, font_size=font_size, color=color,
                        background=background, margin=margin, padding=padding)
        # print(f'svg: {self.svg}\n', ret)
        return ret

    def write(self, text=None):
        if text is not None:
            text = text.replace('\n', '<br>')
            self.element.write(self._message(text), unsafe_allow_html=True)
        return self.element


def test_message():
    pass


if __name__ == '__main__':
    pass



import base64
import cv2
import re


# helpers
# image to base64
def encode_image(image, ext='jpg'):
    try:
        return base64.b64encode(cv2.imencode(f'.{ext}', cv2.imread(image))[1]).decode('utf-8')
    except Exception as e:
        print('image error:', e)
        return ''


def _is_url(url):
    if not url:
        return False
    p = re.compile(
        '^http(s)?://(((([a-z0-9-]{1,10}\.){1,3})([a-z0-9-]{1,3})+)|((\d{1,3}\.)4))(/[a-z0-9-]+)?((\?[a-z0-9-]+=['
        'a-z0-9-.]+)(&[a-z0-9-]+=[a-z0-9%.-]+)*)?$',
        flags=re.I | re.M | re.X)
    return bool(re.match(p, url))


def _is_svg(svg):
    if not svg or not isinstance(svg, str):
        return False
    svg = svg.strip()
    return (svg.startswith('<svg') or svg.startswith('<?xml')) and svg.endswith('</svg>')
